
if (0) {

add_xs

qw(

GdkImlib.xs
Gnome.xs
GnomeAbout.xs
GnomeApp.xs
GnomeCalculator.xs
GnomeCanvas.xs
GnomeCanvasEllipse.xs
GnomeCanvasGroup.xs
GnomeCanvasImage.xs
GnomeCanvasItem.xs
GnomeCanvasLine.xs
GnomeCanvasRE.xs
GnomeCanvasRect.xs
GnomeCanvasText.xs
GnomeClient.xs
GnomeDateEdit.xs
GnomeDialog.xs
GnomeEntry.xs
GnomeFileEntry.xs
GnomeFontSelector.xs
GnomeIconList.xs
GnomeLamp.xs
GnomeLess.xs
GnomeMessageBox.xs
GnomePixmap.xs
GnomePropertyBox.xs
GnomeRootWin.xs
GnomeScores.xs
Gtk.xs
GtkAccelLabel.xs
GtkAccelTable.xs
GtkAdjustment.xs
GtkAlignment.xs
GtkArrow.xs
GtkAspectFrame.xs
GtkBin.xs
GtkBox.xs
GtkButton.xs
GtkButtonBox.xs
GtkCList.xs
GtkCTree.xs
GtkCheckButton.xs
GtkCheckMenuItem.xs
GtkClock.xs
GtkColorSelection.xs
GtkColorSelectionDialog.xs
GtkCombo.xs
GtkContainer.xs
GtkCurve.xs
GtkData.xs
GtkDialog.xs
GtkDrawingArea.xs
GtkEditable.xs
GtkEntry.xs
GtkEventBox.xs
GtkFileSelection.xs
GtkFixed.xs
GtkFontSelection.xs
GtkFontSelectionDialog.xs
GtkFrame.xs
GtkGammaCurve.xs
GtkHBox.xs
GtkHButtonBox.xs
GtkHPaned.xs
GtkHRuler.xs
GtkHScale.xs
GtkHScrollbar.xs
GtkHSeparator.xs
GtkHandleBox.xs
GtkImage.xs
GtkInputDialog.xs
GtkItem.xs
GtkItemFactory.xs
GtkLabel.xs
GtkLed.xs
GtkList.xs
GtkListItem.xs
GtkMenu.xs
GtkMenuBar.xs
GtkMenuItem.xs
GtkMenuShell.xs
GtkMisc.xs
GtkNotebook.xs
GtkObject.xs
GtkOptionMenu.xs
GtkPacker.xs
GtkPaned.xs
GtkPixmap.xs
GtkPlug.xs
GtkPreview.xs
GtkProgress.xs
GtkProgressBar.xs
GtkRadioButton.xs
GtkRadioMenuItem.xs
GtkRange.xs
GtkRuler.xs
GtkScale.xs
GtkScrollbar.xs
GtkScrolledWindow.xs
GtkSelection.xs
GtkSeparator.xs
GtkSocket.xs
GtkSpinButton.xs
GtkStatusbar.xs
GtkTable.xs
GtkTed.xs
GtkTerm.xs
GtkText.xs
GtkTipsQuery.xs
GtkToggleButton.xs
GtkToolbar.xs
GtkTooltips.xs
GtkTree.xs
GtkTreeItem.xs
GtkTty.xs
GtkVBox.xs
GtkVButtonBox.xs
GtkVPaned.xs
GtkVRuler.xs
GtkVScale.xs
GtkVScrollbar.xs
GtkVSeparator.xs
GtkViewport.xs
GtkVtEmu.xs
GtkWidget.xs
GtkWindow.xs
GtkXmHTML.xs
);
}


if ($dver < 1.1) {
	do_package "gtk-1.0";
} else {
	do_package "gtk-1.1";
}
